package com.supervision.manage.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.lang.Assert;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.supervision.exception.BusinessException;
import com.supervision.manage.service.*;
import com.supervision.model.Disease;
import com.supervision.model.MedicalRec;
import com.supervision.service.DiseaseService;
import com.supervision.service.MedicalRecService;
import com.supervision.util.RedisSequenceUtil;
import com.supervision.vo.manage.DiseaseReqVo;
import com.supervision.vo.manage.DiseaseResVo;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

@Service
@RequiredArgsConstructor
public class DiseaseManageServiceImpl implements DiseaseManageService {

    private final DiseaseService diseaseService;

    private final DiseaseAncillaryManageService diseaseAncillaryManageService;

    private final DiseasePhysicalManageService diseasePhysicalManageService;

    private final DiseaseQuestionManageService diseaseQuestionManageService;

    private final DiseaseTreatmentPlanManageService diseaseTreatmentPlanManageService;

    private final MedicalRecService medicalRecService;
    @Override
    public Disease saveDisease(Disease disease) {

        assertDisease(disease);

        // 如果包含疾病id不为空，则为复合疾病
        if (CollUtil.isNotEmpty(disease.getContainDiseaseIds())){
            disease.setDiseaseType(1);
        }
        if (StrUtil.isEmpty(disease.getCode())){
            disease.setCode(RedisSequenceUtil.getComplexDiseaseNo());
        }
        disease.setStatus(0);
        diseaseService.save(disease);

        return disease;

    }

    @Override
    public boolean updateDisease(Disease disease) {

        Assert.isTrue(StrUtil.isNotEmpty(disease.getId()),"id is not allow empty");

        return diseaseService.updateById(disease);

    }

    @Override
    @Transactional
    public boolean deleteDisease(String id) {

        Disease disease = diseaseService.getById(id);
        if (Objects.isNull(disease)){
            return true;
        }
        Assert.isFalse(disease.getDiseaseType()==0,"单一疾病不允许被删除");
        assertDiseaseIsUsed(id);
        // 1. 删除疾病信息
        diseaseService.removeById(id);

        // 2. 删除疾病辅助检查信息
        diseaseAncillaryManageService.deleteByDiseaseId(id);

        // 3. 删除疾病体格检查信息
        diseasePhysicalManageService.deleteByDiseaseId(id);

        //4. 删除疾病问题信息
        diseaseQuestionManageService.deleteByDiseaseId(id);

        // 5. 删除疾病处置计划信息
        diseaseTreatmentPlanManageService.deleteByDiseaseId(id);

        return true;
    }

    @Override
    public IPage<? extends Disease> queryPageList(Integer pageNum, Integer pageSize, DiseaseReqVo diseaseReqVo) {

        Page<Disease> page = diseaseService.lambdaQuery()
                .eq(Objects.nonNull(diseaseReqVo.getDiseaseType()), Disease::getDiseaseType, diseaseReqVo.getDiseaseType())
                .like(StrUtil.isNotEmpty(diseaseReqVo.getDiseaseName()), Disease::getDiseaseName, diseaseReqVo.getDiseaseName())
                .orderByDesc(Disease::getCreateTime).page(new Page<>(pageNum, pageSize));

        if (page.getSize() == 0) {
            return page;
        }
        // 查询已配置病例信息的疾病id
        List<String> medicalDiseaseIds = medicalRecService.lambdaQuery()
                .in(MedicalRec::getDiseaseId, page.getRecords().stream().map(Disease::getId).collect(Collectors.toList()))
                .list().stream().map(MedicalRec::getDiseaseId).distinct().collect(Collectors.toList());

        // 根据diseaseIds查询疾病信息
        List<String> containDiseaseIds = page.getRecords().stream()
                .filter(disease -> CollUtil.isNotEmpty(disease.getContainDiseaseIds()))
                .flatMap(disease -> disease.getContainDiseaseIds().stream()).distinct().collect(Collectors.toList());
        Map<String, Disease> diseaseMap = diseaseService.listByIds(containDiseaseIds)
                .stream().collect(Collectors.toMap(Disease::getId, disease -> disease, (v1, v2) -> v1));

        // 把疾病名映射到DiseaseResVo中
        return page.convert(record -> {
            DiseaseResVo bean = BeanUtil.toBean(record, DiseaseResVo.class);
            if (CollUtil.isNotEmpty(bean.getContainDiseaseIds())){
                bean.setContainDiseaseNames(bean.getContainDiseaseIds().stream()
                        .map(diseaseMap::get).filter(Objects::nonNull).map(Disease::getDiseaseName).collect(Collectors.toList()));
            }
            bean.setUsedMedicalRec(medicalDiseaseIds.contains(bean.getId()));
            return bean;
        });
    }



    private List<Disease> queryByAnyCondition(String code, String diseaseName, String diseaseNameAlias){

        return diseaseService.lambdaQuery().eq(StrUtil.isNotEmpty(code),Disease::getCode,code).or()
                .eq(StrUtil.isNotEmpty(diseaseName), Disease::getDiseaseName,diseaseName).or()
                .eq(StrUtil.isNotEmpty(diseaseNameAlias),Disease::getDiseaseNameAlias,diseaseNameAlias)
                .list();

    }
    private List<Disease> queryByCode(String code){
        if (Objects.isNull(code)){
            throw new BusinessException("code is not allow empty");
        }
        return diseaseService.lambdaQuery().eq(Disease::getCode,code).list();
    }

    private void assertDisease(Disease disease){

        Assert.isNull(disease.getId(),"id 不为空");

        Assert.isTrue(StrUtil.isNotEmpty(disease.getDiseaseName()),"疾病名称不能为空");

        Assert.isTrue(StrUtil.isNotEmpty(disease.getDiseaseNameAlias()),"疾病别名不能为空");

        if (CollUtil.isEmpty(disease.getContainDiseaseIds())){
            // 如果包含疾病为空，则疾病code不能为空
            Assert.isTrue(StrUtil.isNotEmpty(disease.getCode()),"疾病编码不能为空");
        }


    }

    /**
     * 校验疾病是否被使用
     * @param diseaseId 疾病id
     */
    private void assertDiseaseIsUsed(String diseaseId){
        boolean isUsedMedicalRec = medicalRecService.lambdaQuery().eq(MedicalRec::getDiseaseId, diseaseId).count() > 0;
        Assert.isFalse(isUsedMedicalRec,"疾病已经关联病例，不允许删除");
    }

}
